@@ -30,7 +30,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 // z coordinate run the fastest in values, x the slowest   
 
-//#define debug_tools 1
 #define TOOLS_MEMORYERROR 1
 #define TOOLS_XERROR 2
 #define TOOLS_BIN_ERROR 3
@@ -38,7 +37,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 // naechste Zeile nur notwendig, weil atan2 in Windows nicht
 // ordentlich programmiert ist
-#define NEARBYINT(x)  floor((x) * NEARBY + 0.5) / NEARBY
+#define NEARBYINT(x)  (floor((x) * (NEARBY) + 0.5) / (NEARBY))
 // #define NEARBYINT(x)  x
 
 void empvarioXT(double *X, double *T, 
@@ -104,7 +103,15 @@ void empvarioXT(double *X, double *T,
   }
   totalspatialbins =  twoNphiNbin * Ntheta;
   totalbins = totalspatialbins * (nstepT + 1);
-  for (i=0; i<totalbins; i++){sq[i]=sum[i]=0.0; n[i]=0;}
+
+  //printf("total %d %d %d %d \n", nstepT,  twoNphi, *nbin,  Ntheta);
+
+  for (i=0; i<totalbins; i++) { 
+    //printf("%d %d\n", i, totalbins);
+    sq[i] = 0.0;
+    sum[i]= 0.0; 
+    n[i] = 0;
+  }
   for (i=0; i<=*nbin; i++){if (bin[i]>0) BinSq[i]=bin[i] * bin[i]; 
   else BinSq[i]=bin[i];
   }
@@ -298,8 +305,8 @@ void empvarioXT(double *X, double *T,
     sum[i] *= 0.5;
     sq[i] *= 0.25;
   }
-  if (BinSq!=NULL) free(BinSq);
 
+  FREE(BinSq);
   return;
 
  ErrorHandling:
@@ -311,10 +318,10 @@ void empvarioXT(double *X, double *T,
     PRINTF("The x coordinate may not be NULL.\n"); break;
   case TOOLS_BIN_ERROR :
     PRINTF("Bin components not an increasing sequence.\n"); break;
-  default : assert(false);
+  default : BUG;
   }
-  if (BinSq!=NULL) free(BinSq);
-  for (i=0;i<*nbin;i++){sq[i]=sum[i]=RF_NAN;} 
+  FREE(BinSq);
+  for (i=0;i<*nbin;i++){sq[i]=sum[i]=RF_NA;} 
 } 
 	 
 
